! ======================================================================
! Producto escalar DDOT.

! Datos: vectores x, y
! Calculos: producto escalar <x,y>, permite subindices espaciados

! En la funcion DDOT (N, X, INCX, Y, INCY)
!  N es el numero de sumandos del producto escalar,
!  en los vectores X, Y los primeros elementos considerados son PX, PY
!  el sumando i-esimo del producto escalar tiene por factores (?: X o Y)
!    el elemento ?(P? + (i-1)*INC?) si INC?>0 
!    el elemento ?(P? - (N-i)*INC?) si INC?<0
! ======================================================================
 
PROGRAM blas1_ddot
IMPLICIT NONE

INTEGER, PARAMETER:: nmax=1000000, nsal=11
REAL(KIND(0.d0)):: x(nmax), y(nmax)
REAL(KIND(0.d0)):: pef90, peblas
REAL(KIND(0.d0)):: ddot

INTEGER n, incx, incy, nultimo

OPEN (nsal,FILE='BLAS1-DDOT.sal')

! Ejemplo con resultados conocidos

x(10:26:4) = (/ 1.d0, 2.d0, 3.d0, 4.d0, 5.d0 /)
y(40:52:3) = (/ 6.d0, 7.d0, 8.d0, 9.d0, 10.d0 /)

! x(10)y(40) + x(14)y(43) + x(18)y(46) + x(22)y(49) + x(26)y(52) 

pef90 = DOT_PRODUCT(x(10:26:4),y(40:52:3))
peblas = ddot(5, x(10:), 4, y(40:), 3)

WRITE (nsal,'(/(A,G25.17))') 'pef90  = ', pef90, 'peBLAS = ', peblas

! x(26)y(40) + x(22)y(43) + x(18)y(46) + x(14)y(49) + x(10)y(52) 

pef90 = DOT_PRODUCT(x(26:10:-4),y(40:52:3))
peblas = ddot(5, x(10:), -4, y(40:), 3) 

WRITE (nsal,'(/(A,G25.17))') 'pef90  = ', pef90, 'peBLAS = ', peblas

! x(26)y(52) + x(22)y(49) + x(18)y(46) + x(14)y(43) + x(10)y(40) 

pef90 = DOT_PRODUCT(x(26:10:-4),y(52:40:-3))
peblas = ddot(5, x(10:), -4, y(40:), -3) 

WRITE (nsal,'(/(A,G25.17))') 'pef90  = ', pef90, 'peBLAS = ', peblas

! Ejemplo con vectores aleatorios

CALL random_number (x)
CALL random_number (y)

DO incx = 1, 10
  incy = incx
  n = nmax/incx ; nultimo = 1 + incx*(n-1)
  pef90 = DOT_PRODUCT(x(1:nultimo:incx),y(1:nultimo:incy))
  peblas = ddot(n, x, incx, y, incy)
  WRITE (nsal,'(/A,I3)') 'incx = ', incx
  WRITE (nsal,'(A,G25.17)') 'pef90  = ', pef90, 'peBLAS = ', peblas
ENDDO

ENDPROGRAM blas1_ddot