! ======================================================================
! Algebra lineal. Matriz compleja hermitica definida positiva
! ======================================================================

PROGRAM alin_hermitica
IMPLICIT NONE

INTEGER, PARAMETER:: nsal=11, n=3, la=3, lw=64*n, m=2
COMPLEX(KIND(0.d0)):: aa(la,la), bb(n,m), a(la,la), b(n,m), xa(la,la),  &
                       cw(lw), work(lw)
REAL(KIND(0.d0)):: w(n), rwork(3*n)
INTEGER info, i, j
CHARACTER job, uplo

! Primera sentencia ejecutable
! ----------------------------

OPEN (nsal,FILE='HDP-lapack.sal')

! Sistemas a resolver:

!        5x1 + (-1+2i)x2 + (4-i)x3 = 14-37i, 36+13i
! (-1-2i)x1 +       4x2 + (1+i)x3  = 4+i   , -3-19i
!   (4+i)x1 +   (1-i)x2 +    10x3  = 43-60i, 30+20i

aa = RESHAPE( (/ (5.d0,0.d0), (-1.d0,2.d0), (4.d0,-1.d0),               &
                 (-1.d0,-2.d0), (4.d0,0.d0), (1.d0,1.d0),               &
                (4.d0,1.d0), (1.d0,-1.d0), (10.d0,0.d0) /),             &
                 order=(/2,1/), shape=(/3,3/) )
bb = RESHAPE ( (/ (14.d0,-37.d0), (4.d0,1.d0), (43.d0,-60.d0),          &
                  (36.d0,13.d0), (-3.d0,-19.d0), (30.d0,20.d0) /),      &
                  shape=(/3,2/) )

WRITE (nsal,'(T1,A)') 'Matriz a = '
DO i = 1, n
  WRITE (nsal,'(T15,3(:"  (",F20.16,",",F20.16,")")))') aa(i,1:n)
ENDDO

WRITE (nsal,'(/T1,A)') 'RHS = '
DO i = 1, n
  WRITE (nsal,'(T15,2(:"  (",F20.16,",",F20.16,")")))') bb(i,1:m)
ENDDO

! LAPACK

WRITE (nsal,'(/A/23("="))') 'Calculos con LAPACK'

! 1. Descomposicion LU. ZPOTRF (LAPACK)

uplo = 'L'
a = aa
CALL zpotrf (uplo, n, a, la, info)   ! a = LL'
WRITE (nsal,'(/A/29("-")/A,I2)') '1: Descomposicion de Cholesky',       &
       'info =', info
DO i = 1, n
  WRITE (nsal,'(3(:"  (",F20.16,",",F20.16,")")/))') a(i,1:i)
ENDDO

! 2. Sistema AX=B. ZPOTRS (LAPACK)

b = bb
CALL zpotrs (uplo, n, m, a, la, b, la, info)
WRITE (nsal,'(/A/15("-")/A,I2)') '2: Sistema AX=B', 'info =', info
WRITE (nsal,'(T1,A)') 'Solucion = '
DO i = 1, n
  WRITE (nsal,'(T15,2(:"  (",F20.16,",",F20.16,")")))') b(i,1:m)
ENDDO

! 3. Inversa. ZPOTRI (LAPACK)

CALL zpotri (uplo, n, a, la, info)
DO i = 1, n ; a(1:i,i) = CONJG(a(i,1:i)) ; ENDDO   ! U = conjg(L)

WRITE (nsal,'(/A/10("-")/A,I2)') '3: Inversa', 'info =', info
WRITE (nsal,'(T1,A)') 'Inversa de a = '
DO i = 1, n
  WRITE (nsal,'(T15,3(:"  (",F20.16,",",F20.16,")")))') a(i,1:n)
ENDDO

WRITE (nsal,'(/T1,A)') 'a * (a**-1) = '
xa = MATMUL(a,aa)   ! = Identidad
DO i = 1, n
  WRITE (nsal,'(T15,3(:"  (",F20.16,",",F20.16,")")))') xa(i,1:n)
ENDDO

! 4. Autovalores y autovectores. ZHEEV

job = 'V' ; uplo = 'L'
a = aa
CALL zheev (job, uplo, n, a, la, w, work, lw, rwork, info)

WRITE (nsal,'(/A/29("-")/A,I2)') '4: Autovalores y autovectores',       &
       'info = ', info
DO i = 1, n
  WRITE (nsal,'(/T1,A,4X,F20.16)') 'Autovalor: ', w(i)
  WRITE (nsal,'(T1,A,3("  (",F20.16,",",F20.16,")")/))')                &
        'Autovector: ', a(1:n,i)
ENDDO

! Comprobaciones

xa=0
DO i = 1, n ; xa(i,i) = w(i) ; ENDDO  ! diagonal de autovalores

xa = MATMUL(MATMUL(a,xa),CONJG(TRANSPOSE(a)))     ! a
WRITE (nsal,'(/T1,A)') 'a = VDV* = '
DO i = 1, n
  WRITE (nsal,'(T15,3(:"  (",F20.16,",",F20.16,")")))') xa(i,1:n)
ENDDO

xa = MATMUL(a,CONJG(TRANSPOSE(a)))   ! Identidad
WRITE (nsal,'(/T1,A)') 'I = VV* = '
DO i = 1, n
  WRITE (nsal,'(T15,3(:"  (",F20.16,",",F20.16,")")))') xa(i,1:n)
ENDDO

ENDPROGRAM alin_hermitica